@@ -5,6 +5,7 @@ import os
 import stat
 import shutil
 import subprocess
+import re
 from tempfile import NamedTemporaryFile
 
 
@@ -71,17 +72,18 @@ class Deployer:
         return stdout.decode()
 
     @staticmethod
-    def __get_common_prefix_len(path, directory):
+    def __get_common_prefix_len(path, directory, reflective):
         """
         Funzione che controlla se un path ricade all'interno di una directory
-        Non è riflessiva, quindi una cartella non è all'interno di se stessa
+        Se reflective è vero, Non è riflessiva, quindi una cartella non è all'interno di se stessa
         :param path:                path che deve ricadere
         :param directory:           directory nella quale deve ricadere
+        :param reflective           indica se la funzione deve essere riflessiva, ovvero se una cartella viene considerata all'interno di se stessa
         :return:                    la lunghezza del prefisso comune tra il path e la cartella
         """
         # realpath elimina i link simbolici all'interno del path
         directory = os.path.realpath(directory).rstrip('/') + '/'
-        path = os.path.realpath(path)
+        path = os.path.realpath(path) + ('/' if reflective else '')
 
         # controllo se il prefisso comune ai due path
         if os.path.commonprefix([directory, path]) == directory:
@@ -105,11 +107,12 @@ class Deployer:
 
         return os.path.abspath(path)
 
-    def __get_root_folder(self, path):
+    def __get_root_folder(self, path, reflective=False):
         """
         Funzione che verifica se un path ricade tra quelli che sono modificabili e restituisce la directory che matcha
         in modo più specifico
         :param path:            path da verificare
+        :param reflective       indica se la funzione deve considerare le cartelle come riflessive, ovvero se una cartella viene considerata all'interno di se stessa
         :return:                il path all'interno del quale ricade
         """
 
@@ -118,7 +121,7 @@ class Deployer:
 
         # scorro tutte le cartelle che sono modificabili da questo deployer
         for directory in self.__config['folder']:
-            common_prefix_len = self.__get_common_prefix_len(path=path, directory=directory)
+            common_prefix_len = self.__get_common_prefix_len(path=path, directory=directory, reflective=reflective)
             if common_prefix_len > max_common_prefix_len:
                 max_common_prefix_len = common_prefix_len
                 best_matching_folder = directory
@@ -145,6 +148,21 @@ class Deployer:
 
         return perms
 
+    def __get_perms_of_root_folder(self, root_folder):
+        """
+        Funzione che ritorna i permessi definiti per una cartella root presente nella config di questo deployer
+        :param root_folder:         cartella root della quale si vogliono ottenere i permessi
+        :return:                    permessi cartelle, permessi files, uid, gid
+        """
+        dir_perms = self.__config['folder'][root_folder]['dir-perms']
+        file_perms = self.__config['folder'][root_folder]['file-perms']
+
+        # ottengo l'uid dell'owner e il gid del group
+        uid = self.__config['folder'][root_folder]['owner']
+        gid = self.__config['folder'][root_folder]['group']
+
+        return dir_perms, file_perms, uid, gid
+
     def execute_action(self, action_name):
         """
         Esecuzione dell'azione specificata
@@ -200,20 +218,116 @@ class Deployer:
             # errore durante l'eliminazione
             raise RuntimeError('An error occurred deleting file or directory "{}". OSError: {}.'.format(to_delete_path, e))
 
-    def __get_perms_of_root_folder(self, root_folder):
+    def delete_by_regex(self, base_path, regex, inverted, node_type, include_subdirs):
         """
-        Funzione che ritorna i permessi definiti per una cartella root presente nella config di questo deployer
-        :param root_folder:         cartella root della quale si vogliono ottenere i permessi
-        :return:                    permessi cartelle, permessi files, uid, gid
+        Funzione che dato un path elimina il nodo del filesystem corrispondente
+        :param base_path:          path della cartella da cui cominciare ad eliminare
+        :param regex:              regex con cui verificare se eliminare un elemento
+        :param inverted:           booleano che indica se invertire il matche effettuato dalla regex
+        :param node_type:          indica se considerare solo file, solo cartelle o entrambi nell'eliminazione
+        :param include_subdirs:    indica se considerare solo la directory corrente o anche le sottodirectory per le eliminazioni
+
+        :throw PermissionError:     nel caso il base path non rientri in un path modificabile dal deployer
+        :throw RuntimeError:        nel ci siano dei problemi con l'eliminazione del nodo
+        :throw FileNotFoundError:   nel caso il base path non venga trovato
         """
-        dir_perms = self.__config['folder'][root_folder]['dir-perms']
-        file_perms = self.__config['folder'][root_folder]['file-perms']
 
-        # ottengo l'uid dell'owner e il gid del group
-        uid = self.__config['folder'][root_folder]['owner']
-        gid = self.__config['folder'][root_folder]['group']
+        # ottengo il path pulito (rimuovo gli alias e risolvo i link)
+        base_path = self.__resolve_path_aliases(path=base_path)
 
-        return dir_perms, file_perms, uid, gid
+        # controllo se il path è accessibile
+        if self.__get_root_folder(path=base_path, reflective=True) is None:
+            # il path non è modificabile, quindi restituisco errore
+            raise PermissionError('Target base path is located in a not reachable path!')
+
+        # se il path ricade tra quelli modificabili, eseguo l'azione
+
+        try:
+            r = re.compile(regex)
+        except re.error as e:
+            # errore durante la preparazione della regex
+            raise RuntimeError('An error occurred compiling the regex "{}": {}.'.format(regex, e))
+
+        node_type = node_type.lower().strip()
+
+        include_files = (node_type in {'file', 'both'})
+        include_folders = (node_type in {'folder', 'both'})
+
+        errors, too_many_errors = self.__delete_in_folder(base_path, r, inverted, include_files, include_folders, include_subdirs)
+
+        return errors, too_many_errors
+
+
+    def __delete_in_folder(self, base_path, regex, inverted, include_files, include_folders, include_subdirs):
+
+        if self.__get_root_folder(path=os.path.realpath(base_path), reflective=True) is None:
+            # in qualche modo ci troviamo a cercare di eliminare in una directory che è fuori dai path permessi (a causa di un symlink per esempio)
+            # ritorniamo immediatamente poichè non possiamo proseguire
+            return [], False
+
+        errors = []
+        too_many_errors = False
+
+        # gestisco le entry in questa cartella
+        for entry in os.listdir(base_path):
+            # scorro tutte le entry presenti all'interno della cartella
+
+            # ottengo il path assoluto dell'entry
+            entry_abs_path = os.path.join(base_path, entry)
+
+            # salto le entry che non mi interessano
+            if os.path.islink(entry_abs_path) or os.path.ismount(entry_abs_path):
+                continue
+
+            try:
+                if os.path.isdir(entry_abs_path):
+                    # LA ENTRY È UNA DIRECTORY
+
+                    # se le directory devono essere considerate
+                    if include_folders:
+                        # verifico se matcha la regex
+                        m = regex.fullmatch(entry) is not None
+
+                        if (m and not inverted) or (not m and inverted):
+                            # elimino la directory
+                            shutil.rmtree(entry_abs_path)
+
+
+                elif os.path.isfile(entry_abs_path):
+                    # LA ENTRY È UN FILE
+
+                    # se i file devono essere considerati
+                    if include_files:
+                        # verifico se matcha la regex
+                        m = regex.fullmatch(entry) is not None
+
+                        if (m and not inverted) or (not m and inverted):
+                            # elimino il file
+                            os.remove(entry_abs_path)
+
+            except OSError as e:
+                # errore durante l'eliminazione,
+                if not too_many_errors:
+                    errors.append([entry_abs_path, "{}".format(e)])
+                    if len(errors) > 100:
+                        too_many_errors = True
+
+
+        # se vanno considerate anche le sottodirectory
+        if include_subdirs:
+            for d in os.listdir(base_path):
+                if os.path.isdir(os.path.join(base_path, d)):
+                    tmp_errors, tmp_too_many = self.__delete_in_folder(os.path.join(base_path, d), regex, inverted, include_files, include_folders, include_subdirs)
+
+                    # se non abbiamo già raggiunto i 100 errori
+                    if len(errors) < 100:
+                        # aggiungo gli errori fino ad un massimo di 100
+                        errors += tmp_errors[0:100-len(errors)]
+
+                    # too_many_errors è vero se lo era prima oppure se questa iterazione lo riporta come vero
+                    too_many_errors = too_many_errors or tmp_too_many
+
+        return errors, too_many_errors
 
     def create_folder(self, folderpath, overwrite=False, permissions=None):
         """
